"use strict";
/*
* This program and the accompanying materials are made available under the terms of the *
* Eclipse Public License v2.0 which accompanies this distribution, and is available at *
* https://www.eclipse.org/legal/epl-v20.html                                      *
*                                                                                 *
* SPDX-License-Identifier: EPL-2.0                                                *
*                                                                                 *
* Copyright Contributors to the Zowe Project.                                     *
*                                                                                 *
*/
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const imperative_1 = require("@brightside/imperative");
const nls = require("vscode-nls");
const os = require("os");
const fs = require("fs");
const path = require("path");
const url_1 = require("url");
const vscode = require("vscode");
const zowe = require("@brightside/core");
const ProfileLoader = require("./ProfileLoader");
const constants_1 = require("constants");
const localize = nls.config({ messageFormat: nls.MessageFormat.file })(__filename);
let IConnection;
class Profiles {
    constructor(log) {
        this.log = log;
        this.allProfiles = [];
        this.spawnValue = -1;
        this.initValue = -1;
        this.validateAndParseUrl = (newUrl) => {
            let url;
            const validProtocols = ["https"];
            const DEFAULT_HTTPS_PORT = 443;
            const validationResult = {
                valid: false,
                host: null,
                port: null
            };
            try {
                url = new url_1.URL(newUrl);
            }
            catch (error) {
                return validationResult;
            }
            // overkill with only one valid protocol, but we may expand profile types and protocols in the future?
            if (!validProtocols.some((validProtocol) => url.protocol.includes(validProtocol))) {
                return validationResult;
            }
            // if port is empty, set https defaults
            if (!url.port.trim()) {
                validationResult.port = DEFAULT_HTTPS_PORT;
            }
            else {
                validationResult.port = Number(url.port);
            }
            validationResult.host = url.hostname;
            validationResult.valid = true;
            return validationResult;
        };
    }
    static createInstance(log) {
        return __awaiter(this, void 0, void 0, function* () {
            Profiles.loader = new Profiles(log);
            yield Profiles.loader.refresh();
            return Profiles.loader;
        });
    }
    static getInstance() {
        return Profiles.loader;
    }
    loadNamedProfile(name) {
        for (const profile of this.allProfiles) {
            if (profile.name === name && profile.type === "zosmf") {
                return profile;
            }
        }
        throw new Error(localize(0, null)
            + name + localize(1, null));
    }
    getDefaultProfile() {
        return this.defaultProfile;
    }
    refresh() {
        return __awaiter(this, void 0, void 0, function* () {
            if (this.isSpawnReqd() === 0) {
                this.allProfiles = ProfileLoader.loadAllProfiles();
                try {
                    this.defaultProfile = ProfileLoader.loadDefaultProfile(this.log);
                }
                catch (err) {
                    // Unable to load a default profile
                    this.log.warn(localize(2, null) + err.message);
                }
            }
            else {
                const profileManager = new imperative_1.CliProfileManager({
                    profileRootDirectory: path.join(os.homedir(), ".zowe", "profiles"),
                    type: "zosmf"
                });
                this.allProfiles = (yield profileManager.loadAll()).filter((profile) => {
                    return profile.type === "zosmf";
                });
                if (this.allProfiles.length > 0) {
                    this.defaultProfile = (yield profileManager.load({ loadDefault: true }));
                }
                else {
                    ProfileLoader.loadDefaultProfile(this.log);
                }
            }
        });
    }
    listProfile() {
        try {
            this.allProfiles = ProfileLoader.loadAllProfiles();
        }
        catch (error) {
            vscode.window.showErrorMessage(error.message);
        }
        this.allProfiles.map((profile) => {
            return profile.name;
        });
        return this.allProfiles;
    }
    getUrl(urlInputBox) {
        return __awaiter(this, void 0, void 0, function* () {
            return new Promise((resolve) => {
                urlInputBox.onDidHide(() => { resolve(urlInputBox.value); });
                urlInputBox.onDidAccept(() => {
                    if (this.validateAndParseUrl(urlInputBox.value).valid) {
                        resolve(urlInputBox.value);
                    }
                    else {
                        urlInputBox.validationMessage = localize(3, null);
                    }
                });
            });
        });
    }
    createNewConnection(profileName) {
        return __awaiter(this, void 0, void 0, function* () {
            let userName;
            let passWord;
            let zosmfURL;
            let rejectUnauthorize;
            let options;
            const urlInputBox = vscode.window.createInputBox();
            urlInputBox.ignoreFocusOut = true;
            urlInputBox.placeholder = localize(4, null);
            urlInputBox.prompt = localize(5, null);
            urlInputBox.show();
            zosmfURL = yield this.getUrl(urlInputBox);
            urlInputBox.dispose();
            if (!zosmfURL) {
                vscode.window.showInformationMessage(localize(6, null));
                return undefined;
            }
            const zosmfUrlParsed = this.validateAndParseUrl(zosmfURL);
            options = {
                placeHolder: localize(7, null),
                prompt: localize(8, null),
                value: userName
            };
            userName = yield vscode.window.showInputBox(options);
            if (userName === undefined) {
                vscode.window.showInformationMessage(localize(9, null));
                return;
            }
            options = {
                placeHolder: localize(10, null),
                prompt: localize(11, null),
                password: true,
                value: passWord
            };
            passWord = yield vscode.window.showInputBox(options);
            if (passWord === undefined) {
                vscode.window.showInformationMessage(localize(12, null));
                return;
            }
            const quickPickOptions = {
                placeHolder: localize(13, null),
                ignoreFocusOut: true,
                canPickMany: false
            };
            const selectRU = ["True - Reject connections with self-signed certificates",
                "False - Accept connections with self-signed certificates"];
            const ruOptions = Array.from(selectRU);
            const chosenRU = yield vscode.window.showQuickPick(ruOptions, quickPickOptions);
            if (chosenRU === ruOptions[0]) {
                rejectUnauthorize = true;
            }
            else if (chosenRU === ruOptions[1]) {
                rejectUnauthorize = false;
            }
            else {
                vscode.window.showInformationMessage(localize(14, null));
                return undefined;
            }
            for (const profile of this.allProfiles) {
                if (profile.name === profileName) {
                    vscode.window.showErrorMessage(localize(15, null));
                    return undefined;
                }
            }
            IConnection = {
                name: profileName,
                host: zosmfUrlParsed.host,
                port: zosmfUrlParsed.port,
                user: userName,
                password: passWord,
                rejectUnauthorized: rejectUnauthorize
            };
            let newProfile;
            try {
                newProfile = yield this.saveProfile(IConnection, IConnection.name, "zosmf");
            }
            catch (error) {
                vscode.window.showErrorMessage(error.message);
                if (error === constants_1.ECONNREFUSED || error === constants_1.ECONNABORTED) {
                    const reEnterCredentials = "Re-Enter Credentials";
                    vscode.window.showInformationMessage("Re-Enter", reEnterCredentials)
                        .then((selection) => {
                        if (selection === reEnterCredentials) {
                            this.rempromptProfile();
                        }
                    });
                }
            }
            yield zowe.ZosmfSession.createBasicZosmfSession(newProfile);
            vscode.window.showInformationMessage("Profile " + profileName + " was created.");
            return profileName;
        });
    }
    promptCredentials(sessName) {
        return __awaiter(this, void 0, void 0, function* () {
            let userName;
            let passWord;
            let options;
            const loadProfile = this.loadNamedProfile(sessName);
            const loadSession = loadProfile.profile;
            if (!loadSession.user) {
                options = {
                    placeHolder: localize(16, null),
                    prompt: localize(17, null),
                    value: userName
                };
                userName = yield vscode.window.showInputBox(options);
                if (!userName) {
                    vscode.window.showErrorMessage(localize(18, null));
                    return;
                }
                else {
                    loadSession.user = userName;
                }
            }
            if (!loadSession.password) {
                passWord = loadSession.password;
                options = {
                    placeHolder: localize(19, null),
                    prompt: localize(20, null),
                    password: true,
                    value: passWord
                };
                passWord = yield vscode.window.showInputBox(options);
                if (!passWord) {
                    vscode.window.showErrorMessage(localize(21, null));
                    return;
                }
                else {
                    loadSession.password = passWord.trim();
                }
            }
            const updSession = yield zowe.ZosmfSession.createBasicZosmfSession(loadSession);
            return [updSession.ISession.user, updSession.ISession.password, updSession.ISession.base64EncodedAuth];
        });
    }
    rempromptProfile() {
        return __awaiter(this, void 0, void 0, function* () {
            let userName;
            let passWord;
            let options;
            options = {
                placeHolder: localize(22, null),
                prompt: localize(23, null),
                value: userName
            };
            userName = yield vscode.window.showInputBox(options);
            options = {
                placeHolder: localize(24, null),
                prompt: localize(25, null),
                password: true,
                value: passWord
            };
            passWord = yield vscode.window.showInputBox(options);
            const profile = {
                type: "zosmf",
                user: userName,
                password: passWord,
                rejectUnauthorized: false,
                name: ""
            };
            try {
                yield new imperative_1.CliProfileManager({
                    profileRootDirectory: path.join(os.homedir(), ".zowe", "profiles"),
                    type: "zosmf"
                }).update({ profile });
            }
            catch (error) {
                vscode.window.showErrorMessage(error.message);
            }
        });
    }
    saveProfile(ProfileInfo, ProfileName, ProfileType) {
        return __awaiter(this, void 0, void 0, function* () {
            const mainZoweDir = path.join(require.resolve("@brightside/core"), "..", "..", "..", "..");
            // we have to mock a few things to get the Imperative.init to work properly
            try {
                process.mainModule.filename = require.resolve("@brightside/core");
            }
            catch (error) {
                vscode.window.showErrorMessage(error.message);
            }
            try {
                process.mainModule.paths.unshift(mainZoweDir);
            }
            catch (error) {
                vscode.window.showErrorMessage(error.message);
            }
            if (this.initValue === -1) {
                try {
                    // we need to call Imperative.init so that any installed credential manager plugins are loaded
                    yield imperative_1.Imperative.init({ configurationModule: require.resolve("@brightside/core/lib/imperative.js") });
                }
                catch (error) {
                    vscode.window.showErrorMessage(error.message);
                }
                this.initValue = 0;
            }
            let zosmfProfile;
            try {
                zosmfProfile = yield new imperative_1.CliProfileManager({
                    profileRootDirectory: path.join(imperative_1.ImperativeConfig.instance.cliHome, "profiles"),
                    type: "zosmf"
                }).save({ profile: ProfileInfo, name: ProfileName, type: ProfileType });
            }
            catch (error) {
                vscode.window.showErrorMessage(error.message);
            }
            return zosmfProfile.profile;
        });
    }
    isSpawnReqd() {
        if (this.spawnValue === -1) {
            const homedir = os.homedir();
            this.spawnValue = 0;
            try {
                const fileName = path.join(homedir, ".zowe", "settings", "imperative.json");
                const settings = JSON.parse(fs.readFileSync(fileName).toString());
                const value = settings.overrides.CredentialManager;
                this.spawnValue = value !== false ? 0 : 1;
            }
            catch (error) {
                // default to spawn
                this.spawnValue = 0;
            }
        }
        return this.spawnValue;
    }
}
exports.Profiles = Profiles;

//# sourceMappingURL=Profiles.js.map
